use circular_buffer::CircularBuffer;
use core_affinity::CoreId;
use std::mem::MaybeUninit;
use std::sync::Arc;
use std::thread;

const SIGIL: &str = "foobar";
const BUFFER_SIZE: usize = 32;
const ITERATIONS: usize = 10_000_000;

#[test]
fn test_concurrency() {
    let core_ids = match core_affinity::get_core_ids().as_deref() {
        Some(&[_, c1, c2, ..]) => (c1, c2),
        Some(&[c0, c1]) => (c0, c1),
        _ => (CoreId { id: 0 }, CoreId { id: 0 }),
    };
    let mut storage: [MaybeUninit<u8>; BUFFER_SIZE] = [MaybeUninit::uninit(); BUFFER_SIZE];
    let buffer = Arc::new(CircularBuffer::from_storage(&mut storage));
    thread::scope(|s| {
        s.spawn({
            core_affinity::set_for_current(core_ids.0);
            let buffer = Arc::clone(&buffer);
            move || {
                for _ in 0..ITERATIONS {
                    while buffer.push_all(SIGIL.bytes()).is_err() {}
                }
            }
        });

        core_affinity::set_for_current(core_ids.1);
        for _ in 0..ITERATIONS {
            let mut result = String::new();
            while result.len() != SIGIL.len() {
                while buffer.is_empty() {}
                buffer.execute_max(SIGIL.len() - result.len(), |s| {
                    result.push_str(std::str::from_utf8(s).unwrap())
                })
            }
            assert_eq!(SIGIL, &result);
        }
    })
}
